// Created by:  Daniel Sanchez
// Modified by: Tomás Pérez

#include <dirent.h> 
#include <opencv/highgui.h>
#include <opencv/cv.h>
#include "dorientation.h"
#include "imgrotation.h"
#include "test.h"
#include "main.h"


// Override cmp functions for CvPoint and set of CvPoints

inline bool operator<(const CvPoint& lhs, const CvPoint& rhs) {
    return (lhs.x < rhs.x) || (lhs.y < rhs.y);
}

inline bool operator<(const set<CvPoint>& lhs, const set<CvPoint>& rhs) {
    return (lhs.size() < rhs.size());
}

int main() {
    String imageName; // Name of image    
    String maskName; // Name of image    
    IplImage *image = NULL; // Source image    
    IplImage *grabcutImage = NULL; // Source image    
    DIR *dir = opendir(IMAGE_PATH.c_str()); // Points to the directory that contains images    
    struct dirent *ent = NULL; // Struct that represents the image directory
    clock_t startC, finishC; // Start and finish time
    double totalTime = 0;
    std::ostringstream linea;
    FILE *fp;
    string maskSaved;
    cargaModelos();


    if (dir) {
        while ((ent = readdir(dir)) != NULL) {
            // '.' and '..' are not interesting, just the images
            if ((strcmp(ent->d_name, ".") != 0) && (strcmp(ent->d_name, "..") != 0)) {
                startC = clock(); // Start time of image
                // Removing extension from RGB image name
                int lastindex = string(ent->d_name).find_last_of(".");
                string rawname = string(ent->d_name).substr(0, lastindex);
                // Sets image name
                imageName = IMAGE_PATH + string(ent->d_name);
                maskName = MASK_GB_PATH + rawname + MGB_SUFFIX;
                maskSaved = rawname + MGB_SUFFIX;
                cout << imageName.c_str() << endl;
                // Loads image
                image = cvLoadImage(imageName.c_str(), CV_LOAD_IMAGE_COLOR);                
                grabcutImage = cvLoadImage(maskName.c_str(), CV_LOAD_IMAGE_GRAYSCALE);
                if (grabcutImage == NULL){
                    continue;
                }
                int height = grabcutImage -> height;
                int width = grabcutImage -> width;
                int step = grabcutImage -> widthStep / sizeof (uchar);
                uchar* data = (uchar *) grabcutImage->imageData;

                int minX = width; // X min coordinate
                int minY = height; // Y min coordinate
                int maxX = 0; // X max coordinate
                int maxY = 0; // Y max coordinate
                int widthRegion = 0; // region width
                int heightRegion = 0; // region height
                int marginRegion = 0; // margin to center image

                //looking for max min coordinates in mask
                for (int i = 0; i < height; i++) {
                    for (int j = 0; j < width; j++) {
                        if (data[i * step + j ] > 250) {
                            if (minX > j)
                                minX = j;
                            if (maxX < j)
                                maxX = j;
                            if (minY > i)
                                minY = i;
                            if (maxY < i)
                                maxY = i;
                        }
                    }
                }

                // If the image is full black
                if (minX == width && minY == height && maxX == 0 && maxY == 0) {
                    cvReleaseImage(&image);
                    cvReleaseImage(&grabcutImage);
                    continue;
                }

                // calculate width and height from region
                widthRegion = abs(maxX - minX);
                heightRegion = abs(maxY - minY);

                // Apply margins

                if (widthRegion > heightRegion) {
                    marginRegion = heightRegion / 2;
                } else {
                    marginRegion = widthRegion / 2;
                }

                minX = abs(minX - marginRegion);
                minY = abs(minY - marginRegion);
                maxX = abs(maxX + marginRegion);
                maxY = abs(maxY + marginRegion);

                // calculate width and height from region
                widthRegion = abs(maxX - minX);
                heightRegion = abs(maxY - minY);

                //avoid to go out from image
                if (minX < 0) {
                    minX = 0;
                }
                if (minY < 0) {
                    minY = 0;
                }

                if ((minX + widthRegion) > width) {
                    widthRegion = width - minX;
                }

                if ((minY + heightRegion) > height) {
                    heightRegion = height - minY;
                }

                // Calls windowing
                runWindowing(cutRectangle(image, minX, minY, widthRegion, heightRegion), imageName);
                cvSaveImage(maskSaved.c_str(), cutRectangle(grabcutImage, minX, minY, widthRegion, heightRegion));
                // Release memory
                cvReleaseImage(&image);
                cvReleaseImage(&grabcutImage);
                finishC = clock(); // Finish time image

                // Show times                
                totalTime += difftime(finishC, startC) / 1000; // acc total time
                linea << ent->d_name << " TIME (seconds): " << difftime(finishC, startC) / 1000 << "(IMAGE) - " << totalTime << "(TOTAL)" << endl;
                cout << linea.str() << endl;
                fp = fopen("time.log", "a");
                fprintf(fp, linea.str().c_str());
                fclose(fp);
                linea.str("");
                linea.clear();
            }
        }
        // Closes directory
        closedir(dir);
    } else {
        showError("It is not possible to open the directory");
    }
    return EXIT_SUCCESS;
}

/** Show the Image image in a Window with Title title */
void showImage(string title, IplImage * image) {
    cvNamedWindow(title.c_str(), CV_WINDOW_AUTOSIZE); // Create a window for display.    
    cvShowImage(title.c_str(), image); // Show our image inside it.
    cvWaitKey(0);
    cvDestroyWindow(title.c_str());
}
