/*
 * detectcascadas.cpp
 *
 *  Created on: 04/04/2012
 *      Author: Juan Carlos Ortega Pérez
 *      Modified by: Tomás Pérez
 */
#include <iostream>
#include <opencv/cv.h>
#include <opencv/highgui.h>
#include "detectcascadas.h"
#include "svm.h"

using namespace cv;
using namespace std;


char *linis = NULL;
struct svm_node *x;
int max_nr_attr = 324;
static int max_line_len;

void inicializa() {
    x = (struct svm_node *) malloc(max_nr_attr * sizeof (struct svm_node));
}

void exit_input_error(int line_num) {
    fprintf(stderr, "Wrong input format at line %d\n", line_num);
    exit(1);
}

/**
 * Devuelve el vector de tantos elementos como numero de cascadas
 */
void obtenerVector(IplImage *img, int vect[NUM_CAS], svm_model *MODEL[NUM_CAS]) {
    // getting HOG descriptor
    string hog = obtenerHOGfeatures(img);
    inicializa();
    // save every prediction in vect 
    for (int i = 0; i < NUM_CAS; i++) {
        char *data = new char[hog.size() + 1];
        strcpy(data, hog.c_str());
        vect[i] = predict(data, MODEL[i]);
        free(data);
    }
    free(x);
}

/** return the string with HOG features */
string obtenerHOGfeatures(IplImage *img) {
    std::ostringstream linea;
    cv::HOGDescriptor hog; // Use standard parameters here
    vector<float> featureVector;
    cv::Mat imageData(img);
    if (imageData.empty()) {
        featureVector.clear();
        printf("Error: HOG image is empty, features calculation skipped!\n");
        return "";
    }

    hog.winSize.width = 32;
    hog.winSize.height = 32;

    vector<cv::Point> locations;
    hog.compute(imageData, featureVector, cv::Size(8, 8), cv::Size(0, 0), locations);
    // generatting the hog line format
    linea << "0";
    for (int i = 0; i < featureVector.size(); i++) {
        linea << " " << i << ":" << featureVector[i];
    }
    linea << "\n";
    //    FILE *fp;
    //    fp = fopen(".\\salidas\\hog.dat", "a");
    //    fprintf(fp, linea.str().c_str());
    //    fclose(fp);
    imageData.release(); // Release the image again after features are extracted
    return linea.str();
}

/**
 * Muestra el vector resultante
 */
void mostrarVector(int vect[NUM_CAS]) {
    /* Mostramos el vector */
    int i;
    cout << endl << "[";
    for (i = 0; i < NUM_CAS - 1; i++) {
        cout << vect[i] << ", ";
    }
    cout << vect[NUM_CAS - 1] << "]" << endl;
}

/**
 * Redimensiona la imagen al tamaño dado
 * @param src_img
 * @param new_width
 * @param new_height
 * @return 
 */
IplImage* img_resize(IplImage* src_img, int new_width, int new_height) {
    IplImage* des_img;
    des_img = cvCreateImage(cvSize(new_width, new_height), src_img->depth, src_img->nChannels);
    cvResize(src_img, des_img, CV_INTER_LINEAR);
    cvReleaseImage(&src_img);
    return des_img;
}

/*
double predict(char *line, svm_model *model) {
    int i = 0;
    double target_label, predict_label;
    char *idx, *val, *label, *endptr;
    int inst_max_index = -1; // strtol gives 0 if wrong format, and precomputed kernel has <index> start from 0
    
    label = strtok(line, " \t");
    target_label = strtod(label, &endptr);
    if (endptr == label)
        exit(1);

    while (1) {
        if (i >= max_nr_attr - 1) // need one more for index = -1
        {
            max_nr_attr *= 2;
            x = (struct svm_node *) realloc(x, max_nr_attr * sizeof (struct svm_node));
        }

        idx = strtok(NULL, ":");
        val = strtok(NULL, " \t");

        if (val == NULL)
            break;
        errno = 0;
        x[i].index = (int) strtol(idx, &endptr, 10);
        if (endptr == idx || errno != 0 || *endptr != '\0' || x[i].index <= inst_max_index)
            exit(1);
        else
            inst_max_index = x[i].index;

        errno = 0;
        x[i].value = strtod(val, &endptr);
        if (endptr == val || errno != 0 || (*endptr != '\0' && !isspace(*endptr)))
            exit(1);

        ++i;
    }
    x[i].index = -1;
    predict_label = svm_predict(model, x);
    return predict_label;
    
    
}
 * */

// ESTA ES LA BUENA

double predict_file(FILE *input, svm_model *model) {
    int total = 0;
    max_line_len = 1024;
    linis = (char *) malloc(max_line_len * sizeof (char));
    while (readline(input) != NULL) {
        int i = 0;
        char *idx, *val, *label, *endptr;
        int inst_max_index = -1; // strtol gives 0 if wrong format, and precomputed kernel has <index> start from 0

        label = strtok(linis, " \t\n");
        if (label == NULL) // empty line
            exit_input_error(total + 1);

        strtod(label, &endptr);
        if (endptr == label || *endptr != '\0')
            exit_input_error(total + 1);

        while (1) {
            if (i >= max_nr_attr - 1) // need one more for index = -1
            {
                max_nr_attr *= 2;
                x = (struct svm_node *) realloc(x, max_nr_attr * sizeof (struct svm_node));
            }

            idx = strtok(NULL, ":");
            val = strtok(NULL, " \t");

            if (val == NULL)
                break;
            errno = 0;
            x[i].index = (int) strtol(idx, &endptr, 10);
            if (endptr == idx || errno != 0 || *endptr != '\0' || x[i].index <= inst_max_index)
                exit_input_error(total + 1);
            else
                inst_max_index = x[i].index;

            errno = 0;
            x[i].value = strtod(val, &endptr);
            if (endptr == val || errno != 0 || (*endptr != '\0' && !isspace(*endptr)))
                exit_input_error(total + 1);

            ++i;
        }
        x[i].index = -1;
        free(linis);
        return svm_predict(model, x);
    }
}

double predict(char *hog, svm_model *model) {
    int total = 0;
    max_line_len = 1024;
    linis = (char *) malloc(max_line_len * sizeof (char));
    int i = 0;
    char *idx, *val, *label, *endptr;
    int inst_max_index = -1; // strtol gives 0 if wrong format, and precomputed kernel has <index> start from 0

    label = strtok(hog, " \t\n");
    if (label == NULL) // empty line
        exit_input_error(total + 1);

    strtod(label, &endptr);
    if (endptr == label || *endptr != '\0')
        exit_input_error(total + 1);

    while (1) {
        if (i >= max_nr_attr - 1) // need one more for index = -1
        {
            max_nr_attr *= 2;
            x = (struct svm_node *) realloc(x, max_nr_attr * sizeof (struct svm_node));
        }

        idx = strtok(NULL, ":");
        val = strtok(NULL, " \t");

        if (val == NULL)
            break;
        errno = 0;
        x[i].index = (int) strtol(idx, &endptr, 10);
        if (endptr == idx || errno != 0 || *endptr != '\0' || x[i].index <= inst_max_index)
            exit_input_error(total + 1);
        else
            inst_max_index = x[i].index;

        errno = 0;
        x[i].value = strtod(val, &endptr);
        if (endptr == val || errno != 0 || (*endptr != '\0' && !isspace(*endptr)))
            exit_input_error(total + 1);

        ++i;
    }
    x[i].index = -1;
    free(linis);
    return svm_predict(model, x);
}

char* readline(FILE *input) {
    int len;

    if (fgets(linis, max_line_len, input) == NULL)
        return NULL;

    while (strrchr(linis, '\n') == NULL) {
        max_line_len *= 2;
        linis = (char *) realloc(linis, max_line_len);
        len = (int) strlen(linis);
        if (fgets(linis + len, max_line_len - len, input) == NULL)
            break;
    }
    return linis;
}
