/*
 * imgrotation.cpp
 *
 *  Created on: 23/04/2012
 *      Author: Juan Carlos Ortega Pérez
 *      Mail  : juancarloslinux@gmail.com
 */

#include <iostream>
#include <opencv/cv.h>
#include <opencv/highgui.h>
//#include "imgrotation.h"
#include "dorientation.h"

using namespace std;
using namespace cv;

/**
 * Rota una imagen respecto a un punto central (x, y) y un angulo
 */
static IplImage* rotate(IplImage* image, float angle, int x, int y) {

    IplImage *rotatedImage = cvCreateImage(cvSize(image->width, image->height), image->depth, image->nChannels);

    // Asignamos el centro
    CvPoint2D32f center;
    center.x = x;
    center.y = y;
    CvMat *mapMatrix = cvCreateMat(2, 3, CV_32FC1);
    // Matriz de transformacion
    cv2DRotationMatrix(center, angle, 1.0, mapMatrix);
    cvWarpAffine(image, rotatedImage, mapMatrix, CV_INTER_LINEAR + CV_WARP_FILL_OUTLIERS, cvScalarAll(0));

    cvReleaseMat(&mapMatrix);

    return rotatedImage;
}

/**
 * Rota una imagen y despues la corta respecto a un punto (x, y) y
 * un lado (recorta cuadrados)
 */
IplImage* rotateAndCut(IplImage *src, int x, int y, int lado) {
    CvRect region = cvRect(x, y, lado, lado);
    cvSetImageROI(src, region);
    // Reservamos memoria para las imagenes
    IplImage *img_gray = cvCreateImage(cvGetSize(src), IPL_DEPTH_8U, 1);
    IplImage *subimg = cvCreateImage(cvGetSize(src), src->depth, src->nChannels);
    IplImage *res = cvCreateImage(cvSize(lado, lado), src->depth, src->nChannels);

    // Copiamos la region de interes
    cvCopy(src, subimg, NULL);
    cvResetImageROI(src);

    // Pasamos la imagen a escala de grises
    cvCvtColor(subimg, img_gray, CV_RGB2GRAY);

    // Calculamos la orientacion predominante
    double angle = calculateDominantOrientation(img_gray);

    // Rotamos la imagen original en funcion de los grados
    IplImage *rotated = rotate(src, angle, x + (lado / 2), y + (lado / 2));
    cvSetImageROI(rotated, region);

    // Copiamos la region rotada
    cvCopy(rotated, res, NULL);
    cvResetImageROI(res);

    // Liberamos memoria
    cvReleaseImage(&img_gray);
    cvReleaseImage(&subimg);
    cvReleaseImage(&rotated);

    return res;
}

/**
 * Rota una imagen y despues la corta respecto a un punto (x, y) y
 * un lado (recorta cuadrados)
 */
IplImage* cut(IplImage *src, int x, int y, int lado) {
    CvRect region = cvRect(x, y, lado, lado);
    cvSetImageROI(src, region);

    // Reservamos memoria para las imagenes
    IplImage *img_gray = cvCreateImage(cvGetSize(src), IPL_DEPTH_8U, 1);
    IplImage *subimg = cvCreateImage(cvGetSize(src), src->depth, src->nChannels);
    IplImage *res = cvCreateImage(cvSize(lado, lado), src->depth, src->nChannels);

    // Copiamos la region de interes
    cvCopy(src, subimg, NULL);
    cvResetImageROI(src);

    // Pasamos la imagen a escala de grises
    cvCvtColor(subimg, img_gray, CV_RGB2GRAY);

    // Calculamos la orientacion predominante
    double angle = 0.0;

    // Rotamos la imagen original en funcion de los grados
    IplImage *rotated = rotate(src, angle, x + (lado / 2), y + (lado / 2));
    cvSetImageROI(rotated, region);

    // Copiamos la region rotada
    cvCopy(rotated, res, NULL);
    cvResetImageROI(res);

    // Liberamos memoria
    cvReleaseImage(&img_gray);
    cvReleaseImage(&subimg);
    cvReleaseImage(&rotated);

    return res;
}

/**
 * Rota una imagen y despues la corta respecto a un punto (x, y) y
 * un lado (recorta cuadrados)
 */
IplImage* cutRectangle(IplImage *src, int x, int y, int width, int height) {
    CvRect region = cvRect(x, y, width, height);
    cvSetImageROI(src, region);

    // Reservamos memoria para las imagenes
    IplImage *subimg = cvCreateImage(cvGetSize(src), src->depth, src->nChannels);
    IplImage *res = cvCreateImage(cvSize(width, height), src->depth, src->nChannels);
    //
    // Copiamos la region de interes
    cvCopy(src, subimg, NULL);
    cvResetImageROI(src);
    //
    //    // Copiamos la region rotada
    cvCopy(subimg, res, NULL);
    cvResetImageROI(res);

    // Liberamos memoria
    cvReleaseImage(&subimg);
    return res;
}