// Created by: Daniel Sanchez
// Modified by: Tomás Pérez

#ifndef TEST_H_
#define TEST_H_

using namespace cv;
using namespace std;

// Constants
const int N_ROWS = 6; // Size ECOC rows
const int N_COLUMNS = 5; // Size ECOC columns
const int SIZE = 20; // Initial size for sliding window
const int LIMIT = 70; // Limit region size
const int STEP = 2; // Distance between regions

const float GEOMETRIC_FACTOR = 1.1; // Multiplied by the region side to increase region size         

const String IMAGE_PATH = ".\\img\\"; // Path name for images  
const String MAP_PATH = ".\\map\\"; // Path name for result map
const String MASK_GB_PATH = "..\\maskGB\\"; // Path name for grabcut masks

const String EXT = ".bmp"; // Image extension
const String MGB_SUFFIX = "_maskGrabCut.bmp"; // Suffix to name grab cut mask
const String MP_SUFFIX = ".png"; // Suffix to name image of probability

// Model files
const String modelFiles[] = {
    "..\\MODEL\\EXP9_H1.model",
    "..\\MODEL\\EXP9_H2.model",
    "..\\MODEL\\EXP9_H3.model",
    "..\\MODEL\\EXP9_H4.model",
    "..\\MODEL\\EXP9_H5.model"
};

// Types of articulation
const String TYPE[N_ROWS] = {"torso", "cabeza", "brazos", "antebrazos", "muslos", "piernas"};

const String MASK_ID[N_ROWS] = {"2", "1", "87", "65", "1413", "1211"};

// ECOC matrix
const int ECOC[N_ROWS][N_COLUMNS] = {
    {1, 1, 0, 0, 0},
    {1, -1, 0, 0, 0},
    {-1, 0, 1, 1, 0},
    {-1, 0, 1, -1, 0},
    {-1, 0, -1, 0, 1},
    {-1, 0, -1, 0, -1}
};

// Weights for ECOC decoding
const float WEIGHTS[N_COLUMNS] = {0.878578, 0.969844, 0.781783,
    0.749476, 0.796539};

// Structs

struct EcocData {
    float confidences[N_ROWS];
    int row[N_COLUMNS];
    String name;
};

struct RegionData {
    int posX;
    int posY;
    int size;
    struct EcocData data;
};

struct ProbData {
    CvPoint previousPos;
    CvPoint currentPos;
    CvPoint meanPos;
    int nRegions;
    int countSize;
    int meanSize;
    int distance;
};

// Function definitions
String extractName(String path);
void showError(const char *error);
void computeBlackImage(IplImage *image);
void getArticulation(struct EcocData *data);
void getBinMask(const Mat& comMask, Mat& binMask);
void runWindowing(IplImage *image, String imageName);
void setPixel(int type, int posY, int posX, IplImage *image);
void computeEcocData(int currentRow[], struct EcocData *data);
void setGraphCutPixel(int type, int posY, int posX, IplImage *image);
void runGrabCut(IplImage *image, IplImage *grabCutImage, String imageName);
void computeProbability(IplImage *image, RegionData *region, ProbData *probData);
void computeProbabilities(IplImage *image, vector<RegionData> *regions, String imageName);
void generateImage(Mat& _img, Mat& _mask, vector<Point>& _bgdPxls, vector<Point>& _fgdPxls, set<CvPoint> unionCompute, IplImage *image, IplImage *result, float *max, double *sf1, double *pf1, double *pp1, double *sp1, double sf, double pf, double pp, double sp);
char* readline(FILE *input);
void cargaModelos();

void inicializa();

#endif	/* TEST_H_ */